{% load wagtailcore_tags %}

<div class="live-post" data-post-id="{{block_id}}">
    <div class="live-post-meta">
        <div class="live-post-created">{{ value.created }}</div>
        {% if value.modified %}
            <div class="live-post-modified">Edited on: {{ value.modified }}</div>
        {% endif %}
    </div>
    <div class="live-post-content">
        {% for block in value.content %}
            <div class="post-block">{% include_block block %}</div>
        {% endfor %}
    </div>
</div>